use super::structs::Street;
use crate::errors::FejError;
use reqwest::blocking as reqwest;
use std::collections::HashMap;
use std::convert::TryFrom;

/// Endpoint for the search feature
const SEARCH_URL: &str = "https://www.ivago.be/nl/particulier/autocomplete/garbage/streets";

/// Searches the Ivago API for streets in the given city
///
/// # Arguments
///
/// * `street` - name of the street
/// * `city` - city the street is in
// TODO find out how to do this async
pub fn search_streets(street_name: &str) -> Result<Vec<Street>, FejError> {
    let client = reqwest::Client::new();
    let response = client.get(SEARCH_URL).query(&[("q", street_name)]).send()?;
    let data: Vec<HashMap<String, String>> = response.json()?;

    let mut output: Vec<Street> = Vec::new();

    // We iterate over every item and extract the needed data
    for map in data.iter() {
        if let Some(value) = map.get("value") {
            match Street::try_from(value.as_str()) {
                Ok(street) => output.push(street),
                Err(_) => continue,
            }
        }
    }

    Ok(output)
}
